module sim;
    reg clk = 0;
    wire coreclk;
    reg memclk = 0;
	reg reset = 1;
	always #2 clk = !clk;

    phoenixClockDividerBy2 corediv(clk, coreclk);

    wire [7:0] data_bus;
    wire [15:0] address_bus;

    wire mem_write;
    wire mem_read;

    wire io_out;
    wire io_in;

    wire [7:0] gpio_out;
    wire [7:0] gpio_in = 8'h69;
    
    reg int0 = 1'b0;
    reg int1 = 1'b0;
    reg int2 = 1'b0;
    reg int3 = 1'b0;
    reg int4 = 1'b0;
    wire int5;

	reg prog_din = 1'b0;
	reg prog_clk = 1'b0;
	reg prog_latch = 1'b0;

    wire timer_out;

    wire tx_pin;

    reg [31:0] cycles = 0;
    always @(posedge clk ) begin
        cycles <= cycles + 1;
    end

    phoenix soc (
        coreclk,
        reset,
        data_bus,
        address_bus,
        mem_write,
        mem_read,
        io_out,
        io_in,
        gpio_out,
        gpio_in,
        int0,
        int1,
        int2,
        int3,
        int4,
        int5,
        timer_out
    );

    phoenixRam ram (
        clk,
        mem_read,
        mem_write,
        data_bus,
        address_bus,
        prog_din,
	    prog_clk,
	    prog_latch
    );


    phoenixUart uart (
        .clk(coreclk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .tx_pin(tx_pin),
        .rx_pin(tx_pin),
        .uart_int(int5),
        .uart_clk(clk)
    );

    always @(posedge clk) begin
        if (soc.core.halt) begin      
        `ifdef MEMFILE
            $writememh(`MEMFILE, ram.memory);
        `else
            $writememh("mem.hex", ram.memory);
        `endif
            $display("Took %d cycles", cycles);
            $finish;
        end
    end

    initial begin
    `ifdef DUMPFILE
		$dumpfile(`DUMPFILE);
    `else
        $dumpfile("dump.lxt");
    `endif

        $dumpvars(0, sim);

        #11 reset = 0;
    end

endmodule