module phoenixAlu (
    input clk,
    input reset,

    input SAVE_DB_ALUA,
    input SAVE_DB_ALUB,

    input PUT_ALUOUT_DB,

    input ALU_ADD,
    input ALU_SUB,
    input ALU_NAND,
    input ALU_NOR,
    input ALU_ADO,
    input ALU_SBO,
    input ALU_CMP,

    input PUT_FG_DB,
    input PUT_DB_FG,
    input CLEAR_FG,

    inout [7:0] data_bus,

    output reg [7:0] fg
);

    reg [7:0] a;
    reg [7:0] b;

    reg [8:0] out;


    wire [7:0] data_bus_int = (
        PUT_ALUOUT_DB ? out : 8'd0 |
        PUT_FG_DB ? fg : 8'd0
    );
    assign data_bus = (
        PUT_ALUOUT_DB ||
        PUT_FG_DB
    ) ? data_bus_int : 8'bzzzzzzzz;

    always @(negedge clk or posedge reset) begin
        if (reset) begin
            a <= 8'd0;
        end else begin
            if (SAVE_DB_ALUA) begin
                a <= data_bus;
            end
        end
    end

    always @(negedge clk or posedge reset) begin
        if (reset) begin
            b <= 8'd0;
        end else begin
            if (SAVE_DB_ALUB) begin
                b <= data_bus;
            end
        end
    end

    always @(negedge clk or posedge reset) begin
        if (reset) begin
            fg <= 8'd0;
            out <= 9'd0;
        end else begin
            if (ALU_ADD) begin
                out = a + b;
            end
            if (ALU_SUB) begin
                out = a - b;
            end
            if (ALU_NAND) begin
                out = ~(a & b);
            end
            if (ALU_NOR) begin
                out = ~(a | b);
            end
            if (ALU_ADO) begin
                if (fg[2]) begin
                    out = a + b;
                end else begin
                    out = a;
                end
            end
            if (ALU_SBO) begin
                if (fg[2]) begin
                    out = a - b;
                end else begin
                    out = a;
                end
            end
            if (ALU_CMP) begin
                fg[0] = a == 8'd0;
                fg[1] = a == b;
            end
            if (PUT_DB_FG) begin
                fg = data_bus;
            end
            if (CLEAR_FG) begin
                fg = 8'd0;
            end
            if (out[8]) begin
                fg[2] <= 1'b1;
            end
        end
    end

endmodule