module phoenixRegisters (
    input clk,

    input PUT_A_ADDR,
    input PUT_B_ADDR,

    input PUT_IR0_DB,
    input PUT_IR1_DB,

    input SAVE_ADDR_A,
    input SAVE_ADDR_B,

    input SAVE_DB_IR0,
    input SAVE_DB_IR1,

    inout [7:0] data_bus,
    inout [15:0] address_bus,

    input [3:0] ir0,
    input [3:0] ir1
);
    reg [7:0] registers [0:15];
    
    wire [15:0] address_bus_int = (
        PUT_A_ADDR ? {registers[1], registers[0]} : 16'd0 |
        PUT_B_ADDR ? {registers[3], registers[2]} : 16'd0
    );
    assign address_bus = (
        PUT_A_ADDR ||
        PUT_B_ADDR
    ) ? address_bus_int : 16'bzzzzzzzzzzzzzzzz;

    wire [7:0] data_bus_int = (
        PUT_IR0_DB ? registers[ir0] : 8'd0 |
        PUT_IR1_DB ? registers[ir1] : 8'd0
    );
    assign data_bus = (
        PUT_IR0_DB ||
        PUT_IR1_DB
    ) ? data_bus_int : 8'bzzzzzzzz;

    always @(negedge clk) begin
        if (SAVE_DB_IR0) begin
            registers[ir0] <= data_bus;
        end
        if (SAVE_DB_IR1) begin
            registers[ir1] <= data_bus;
        end
        if (SAVE_ADDR_A) begin
            {registers[1], registers[0]} <= address_bus;
        end
        if (SAVE_ADDR_B) begin
            {registers[3], registers[2]} <= address_bus;
        end
    end
endmodule