module phoenix (
    input clk,
    input reset,

    inout [7:0] data_bus,
    inout [15:0] address_bus,

    output mem_write,
    output mem_read,

    output io_out,
    output io_in,

    output [7:0] gpio_out,
    input [7:0] gpio_in,

    input int0,
    input int1,
    input int2,
    input int3,
    input int4,
    input int5,

    output timer_out
);
    wire interrupt_trigger;
    wire interrupt_done;

    wire software_interrupt;

    wire timer_interrupt;

    wire [7:0] interrupt = {software_interrupt, timer_interrupt, int5, int4, int3, int2, int1, int0};

    phoenixCore core (
        .extClk(clk),
        .reset(reset),
        .data_bus(data_bus),
        .address_bus(address_bus),
        .interrupt_trigger(interrupt_trigger),
        .interrupt_done(interrupt_done),
        .SOFTWARE_INTERRUPT(software_interrupt),
        .mem_write(mem_write),
        .mem_read(mem_read),
        .IO_OUT(io_out),
        .IO_IN(io_in)
    );

    phoenixInterruptController interrupt_controller (
        .clk(clk),
        .reset(reset),
        .interrupt_done(interrupt_done),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .interrupt_in(interrupt),
        .trigger(interrupt_trigger)
    );

    phoenixTimer timer (
        .clk(clk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .timer_interrupt(timer_interrupt),
        .timer_out(timer_out)
    );

    phoenixIo gpio (
        .clk(clk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .port_in(gpio_in),
        .port_out(gpio_out),
        .clear(1'b0)
    );
    
endmodule