module phoenixTimer (
    input clk,
    input reset,

    input io_in,
    input io_out,

    input [15:0] address_bus,
    inout [7:0] data_bus,

    output reg timer_interrupt,
    output reg timer_out
);
    parameter base_address = 16'hff01;

    reg [7:0] prescaler_count;
    reg [7:0] timer_count;


    wire [7:0] prescaler_out;
    phoenixIo prescaler (
        .clk(clk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .port_in(prescaler_out),
        .port_out(prescaler_out),
        .clear(1'b0)
    );
    defparam prescaler.address = base_address;

    wire [7:0] compare_out;
    phoenixIo compare (
        .clk(clk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .port_in(compare_out),
        .port_out(compare_out),
        .clear(1'b0)
    );
    defparam compare.address = base_address + 1;

    wire [7:0] control_out;
    phoenixIo control (
        .clk(clk),
        .reset(reset),
        .io_in(io_in),
        .io_out(io_out),
        .address_bus(address_bus),
        .data_bus(data_bus),
        .port_in(control_out),
        .port_out(control_out),
        .clear(1'b0)
    );
    defparam control.address = base_address + 2;
    
    wire enable = control_out[0];
    wire prescaler_enable = control_out[1];
    wire interrupt_enable = control_out[2];
    wire pulse_out = control_out[3];
    wire toggle_out = control_out[4];

    always @(posedge clk or posedge reset) begin
        if (reset) begin
            prescaler_count <= 8'd0;
            timer_count <= 8'd0;
            timer_out <= 1'b0;
            timer_interrupt <= 1'b0;
        end else begin
            if (enable) begin     
                if (prescaler_enable) begin
                    prescaler_count <= prescaler_count + 1;
                    if (prescaler_count == prescaler_out) begin
                        timer_count <= timer_count + 1;
                        prescaler_count <= 8'd0;
                    end
                end else begin
                    timer_count <= timer_count + 1;
                end

                if (timer_count == compare_out) begin
                    timer_count <= 8'd0;

                    if (toggle_out) begin
                        timer_out <= !timer_out; 
                    end
                    if (pulse_out) begin
                        timer_out <= 1'b1;
                    end
                    if (interrupt_enable) begin
                        timer_interrupt <= 1'b1;
                    end
                end else begin
                    if (pulse_out) begin
                        timer_out <= 1'b0;
                    end
                    timer_interrupt <= 1'b0;
                end
            end
        end
    end

endmodule